package com.pmc.loganalysis.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * @author
 */
public class DateUtils {

    private final static Logger logger = LoggerFactory.getLogger(DateUtils.class);

    public static String dateToStr(Date d, String format) {
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        return sdf.format(d);
    }

    /**
     * 时间格式(yyyy-MM-dd)
     */
    public final static String DATE_PATTERN = "yyyy-MM-dd";
    /**
     * 时间格式(yyyy/MM/dd)
     */
    public final static String DATE_XIE = "yyyy/MM/dd";

    public final static String DATE_YYYYMMDD = "yyyyMMdd";
    public final static String DATE_YYYYMMDDHH = "yyyyMMddHH";
    public final static String DATE_YYYYMMDDHHMM = "yyyyMMddHHmm";

    public final static String DATE_YYYYMMDDHHMMSS = "yyyyMMddHHmmss";

    public final static String DATE_YYYYMMDDHHMMSSSSS = "yyyyMMddHHmmssSSS";

    /**
     * 时间格式(yyyy-MM-dd HH:mm:ss)
     */
    public final static String DATE_TIME_PATTERN = "yyyy-MM-dd HH:mm:ss";
    /**
     * 时间格式(yyyy-MM-dd HH:mm:ss)
     */
    public final static String DATE_TIME_PATTERNS = "yyyy-MM-dd HH:mm";

    /**
     * 时间格式(yyyy-MM-dd HH:mm:ss.SSS)
     */
    public final static String DATE_TIME_PATTERN_YYYYMMDDHHMMSSSSS = "yyyy-MM-dd HH:mm:ss.SSS";

    public static String format(Date date) {
        return format(date, DATE_PATTERN);
    }

    public static String format(Date date, String pattern) {
        if (date != null) {
            SimpleDateFormat df = new SimpleDateFormat(pattern);
            return df.format(date);
        }
        return null;
    }

    public static String format2() {
        return DateUtils.format(DateUtils.getNow(), DateUtils.DATE_TIME_PATTERN);
    }

    public static String format2(Date date) {
        return DateUtils.format(date, DateUtils.DATE_TIME_PATTERN);
    }

    public static String format3(long time) {
        SimpleDateFormat sdf = new SimpleDateFormat(DATE_TIME_PATTERN_YYYYMMDDHHMMSSSSS);
        return sdf.format(time);
    }

    /**
     * 计算距离现在多久，非精确
     *
     * @param date
     * @return
     */
    public static String getTimeBefore(Date date) {
        Date now = new Date();
        long l = now.getTime() - date.getTime();
        long day = l / (24 * 60 * 60 * 1000);
        long hour = (l / (60 * 60 * 1000) - day * 24);
        long min = ((l / (60 * 1000)) - day * 24 * 60 - hour * 60);
        long s = (l / 1000 - day * 24 * 60 * 60 - hour * 60 * 60 - min * 60);
        String r = "";
        if (day > 0) {
            r += day + "天";
        } else if (hour > 0) {
            r += hour + "小时";
        } else if (min > 0) {
            r += min + "分";
        } else if (s > 0) {
            r += s + "秒";
        }
        r += "前";
        return r;
    }

    /**
     * 计算距离现在多久，精确
     *
     * @param date
     * @return
     */
    public static String getTimeBeforeAccurate(Date date) {
        Date now = new Date();
        long l = now.getTime() - date.getTime();
        long day = l / (24 * 60 * 60 * 1000);
        long hour = (l / (60 * 60 * 1000) - day * 24);
        long min = ((l / (60 * 1000)) - day * 24 * 60 - hour * 60);
        long s = (l / 1000 - day * 24 * 60 * 60 - hour * 60 * 60 - min * 60);
        String r = "";
        if (day > 0) {
            r += day + "天";
        }
        if (hour > 0) {
            r += hour + "小时";
        }
        if (min > 0) {
            r += min + "分";
        }
        if (s > 0) {
            r += s + "秒";
        }
        r += "前";
        return r;
    }

    /**
     * 获得当前日期
     *
     * @return
     */
    public static Date getNow() {
        Calendar cal = Calendar.getInstance();
        Date currDate = cal.getTime();
        return currDate;
    }

    //    字符串转化成日期
    public static Date parse(String dateStr, String pattren) {
        try {
            return new SimpleDateFormat(pattren).parse(dateStr);
        } catch (ParseException e) {
            logger.error("字符串转化成日期失败", e);
        }
        return null;
    }

    /**
     * date2比date1多的天数
     *
     * @param date1
     * @param date2
     * @return
     */
    public static int differentDays(Date date1, Date date2) {
        Calendar calendar = Calendar.getInstance();
        calendar.clear();
        calendar.setTime(date1);
        int day1 = calendar.get(Calendar.DAY_OF_YEAR);
        int year1 = calendar.get(Calendar.YEAR);
        calendar.setTime(date2);
        int day2 = calendar.get(Calendar.DAY_OF_YEAR);
        int year2 = calendar.get(Calendar.YEAR);
        //同一年
        if (year1 == year2) {
            return day2 - day1;
        } else if (year1 < year2) {
            //Date1<Date2
            int days = 0;
            for (int i = year1; i < year2; i++) {
                if (i % 4 == 0 && i % 100 != 0 || i % 400 == 0) {
                    //闰年
                    days += 366;
                } else {
                    days += 365;
                }
            }
            return days + (day2 - day1);
        } else {//Date1>Date2
            int days = 0;
            for (int i = year2; i < year1; i++) {
                if (i % 4 == 0 && i % 100 != 0 || i % 400 == 0) {
                    days += 366;
                } else {
                    days += 365;
                }
            }
            return 0 - days + (day2 - day1);
        }
    }

    public static int compareDate(Date dt1, Date dt2) {
        if (dt1.getTime() > dt2.getTime()) {
            return 1;
        } else if (dt1.getTime() < dt2.getTime()) {
            return -1;
        } else {
            return 0;
        }
    }

    /**
     * 获取时间
     *
     * @param pattern 格式化后的样式
     * @param number  分钟数  -5 五分钟前  0 现在  5 五分钟后
     * @return
     */
    public static String getDate(String pattern, int number) {
        Calendar now = Calendar.getInstance();
        now.add(Calendar.MINUTE, number);
        Date time = now.getTime();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(pattern);
        return simpleDateFormat.format(time);
    }

    /**
     * 获取时间
     *
     * @param number 分钟数  -5 五分钟前  0 现在  5 五分钟后
     * @return
     */
    public static Date getDate(Date now, int number) {
        //一秒是1000
        //一分钟是60000
        Date dateTime = new Date(now.getTime() + number * 60000);
        return dateTime;
    }

    /**
     * 获取指定时间的年月日时分秒字符串
     * @param date 日期
     * @param format 格式
     * @return 结果
     */
    public static String getDateStringYMDHMS(Date date, String format) {
        String sdate = "";
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        sdate = sdf.format(date);
        return sdate;
    }

    /**
     * 转换时间字符串为Date,自定义格式
     * @param sdate
     * @param format
     * @return
     */
    public static Date getDateByString(String sdate, String format) {
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        Date date = null;
        try {
            date = sdf.parse(sdate);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }
    /**
     * 将日期转换成时间戳
     * @param sdate
     * @param
     * @return
     */
    public static long getcurrentTimeMillisByDate(String sdate, String format) {
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        long date = 0;
        try {
            date = sdf.parse(sdate).getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }

    public static String getDateBycurrentTimeMillis(long sdate, String format) {
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        String date = sdf.format(sdate);
        return date;
    }

    //获取当前时间的00：00时

    public static Date getDateByday() {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MINUTE,0);
        Date zero = calendar.getTime();
        return zero;
    }
}
