
package com.pmc.loganalysis.util;

import com.alibaba.fastjson.JSONObject;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.HttpStatus;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Map;

// Http请求的工具类
@Slf4j
public class HttpUtils {

	private static RequestConfig requestConfig = null;
	static {
		// 设置请求和传输超时时间
		requestConfig = RequestConfig.custom().setSocketTimeout(10000).setConnectTimeout(10000).build();
	}

	/**
	 * post请求传输json参数
	 * 
	 * @param url  url地址
	 * @param
	 * @return
	 */
	public static Map<String,Object> httpPost(String url, JSONObject jsonParam) {
		Map<String,Object> map=new HashMap();
		map.put("repStr","");
		// post请求返回结果
		CloseableHttpClient httpClient = HttpClients.createDefault();
		String str = "";
		HttpPost httpPost = new HttpPost(url);
		// 设置请求和传输超时时间
		httpPost.setConfig(requestConfig);
		CloseableHttpResponse result=null;
		try {
			if (null != jsonParam) {
				// 解决中文乱码问题
				StringEntity entity = new StringEntity(jsonParam.toString(), "utf-8");
				entity.setContentEncoding("UTF-8");
				entity.setContentType("application/json");
				httpPost.setEntity(entity);
			}
			result = httpClient.execute(httpPost);
			// 请求发送成功，并得到响应
			if (result.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
				try {
					// 读取服务器返回过来的json字符串数据
					str = EntityUtils.toString(result.getEntity(), "utf-8");
					map.put("repStr",str); // 服务器响应为200，才会存入 str
				} catch (Exception e) {
					// 请求和返回正常，但是解析数据时有问题
					log.error("post请求提交失败:" + url, e);
				}
			}
			String statusCode=result.getStatusLine().getStatusCode()+"";
			map.put("statusCode",statusCode);
		} catch (Exception e) {
			log.error("post请求提交失败:" + url, e);
			map.put("statusCode","400");
		} finally {
			httpPost.releaseConnection();
		}
		return map;
	}

	/**
	 * 发起webservice请求
	 * @param url
	 * @param soap
	 * @param SOAPAction
	 * @return
	 */
	public static Map<String,Object> doPostSoap1(String url, String soap, String SOAPAction) {
		Map<String,Object> map=new HashMap();
		map.put("repStr","");
		//请求体
		String retStr = "";
		// 创建HttpClientBuilder
		HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
		// HttpClient
		CloseableHttpClient closeableHttpClient = httpClientBuilder.build();

		HttpPost httpPost = new HttpPost(url);
		//  设置请求和传输超时时间
		RequestConfig requestConfig = RequestConfig.custom()
				.setSocketTimeout(60000)
				.setConnectTimeout(60000).build();
		httpPost.setConfig(requestConfig);
		try {
			httpPost.setHeader("Content-Type", "application/xml;charset=UTF-8");
			httpPost.setHeader("SOAPAction", SOAPAction);
			StringEntity data = new StringEntity(soap,
					Charset.forName("UTF-8"));
			httpPost.setEntity(data);
			CloseableHttpResponse response = closeableHttpClient
					.execute(httpPost);
			log.info("HttpStatus  {}",response.getStatusLine().getStatusCode());
			if(response.getStatusLine().getStatusCode() == HttpStatus.SC_OK){
				HttpEntity httpEntity = response.getEntity();
				if (httpEntity != null) {
					// 打印响应内容
					retStr = EntityUtils.toString(httpEntity, "UTF-8");
					map.put("repStr",retStr);

					log.info("doPostSoap1 retStr {}",retStr);
				}
			}
			String statusCode=response.getStatusLine().getStatusCode()+"";
			map.put("statusCode",statusCode);
			// 释放资源
			closeableHttpClient.close();
		} catch (Exception e) {
			map.put("statusCode","400");
			e.printStackTrace();
			log.error("doPostSoap1 Exception {}",e);
		}
		return map;
	}


	/**
	 * 发起webservice请求
	 * @param url
	 * @param soap
	 * @param SOAPAction
	 * @return
	 */
	public static String doPostSoap(String url, String soap, String SOAPAction) {
		//请求体
		String retStr = "";
		// 创建HttpClientBuilder
		HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
		// HttpClient
		CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
		HttpPost httpPost = new HttpPost(url);
		//  设置请求和传输超时时间
		RequestConfig requestConfig = RequestConfig.custom()
				.setSocketTimeout(60000)
				.setConnectTimeout(60000).build();
		httpPost.setConfig(requestConfig);
		try {
			httpPost.setHeader("Content-Type", "application/xml;charset=UTF-8");
			httpPost.setHeader("SOAPAction", SOAPAction);
			StringEntity data = new StringEntity(soap,
					Charset.forName("UTF-8"));
			httpPost.setEntity(data);
			CloseableHttpResponse response = closeableHttpClient
					.execute(httpPost);
			HttpEntity httpEntity = response.getEntity();
			if (httpEntity != null) {
				// 打印响应内容
				retStr = EntityUtils.toString(httpEntity, "UTF-8");
				log.info("短信网关响应结果response:" + retStr);
			}
			// 释放资源
			closeableHttpClient.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return retStr;
	}
	
	@Data
	public
	class ResponseValue {
		private int status;
		private String value;
		private String resultCode;
		private String resultDesc;
	}

}
