package com.pmc.loganalysis.util.shell;


import ch.ethz.ssh2.Connection;
import com.jcraft.jsch.*;
import ch.ethz.ssh2.Session;
import ch.ethz.ssh2.StreamGobbler;
import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.io.*;
import java.util.Calendar;

@Slf4j
public class ExecShell {


    public static void shell(String cmd) {
        try {
            Runtime.getRuntime().exec(cmd);
        } catch (IOException e) {
            log.error("shell 执行失败" + cmd, e);
        }

    }

    /**配置连接
     * @param user
     * @param passwd
     * @param host
     * @throws Exception
     * @return
     */
    public static com.jcraft.jsch.Session connect(String user, String passwd, String host) throws Exception {
        JSch jsch = new JSch();
        com.jcraft.jsch.Session session = jsch.getSession(user, host);
        if (session == null) {
            throw new Exception("session is null");
        }
        session.setPassword(passwd);
        java.util.Properties config = new java.util.Properties();
        //第一次登陆
        config.put("StrictHostKeyChecking", "no");
        config.put("Ciphers","3des-cbc");
        session.setConfig(config);
        log.info("登录主机host:{},user:{}",host,user);
        try {
            session.connect(30000);
        } catch (Exception e) {
            log.info("连接远程服务器失败 host:{},user:{}",host,user);
            throw new Exception("连接远程端口无效或用户名密码错误");
        }
        return session;
    }


    private static String DEFAULT_CHAR_SET = "UTF-8";
    private static String tipStr = "=======================%s=======================";
    private static String splitStr = "=====================================================";



    /**
     * 执行shell脚本或者命令
     * @param cmd
     *  即将执行的命令
     * @return
     *  命令执行完后返回的结果值
     */
    public static String executes( String cmd){

        StringBuffer buffer = new StringBuffer();
        BufferedReader reader = null;
        try {
            Process proc = Runtime.getRuntime().exec(cmd);
            InputStream in = proc.getInputStream();
            reader = new BufferedReader(new InputStreamReader(in));
            String line = null;
            while ((line = reader.readLine()) != null) {
                buffer.append(line);
            }
            int exitVal = proc.waitFor();
            log.info("执行命令返回结果exitVal："+exitVal);
        }catch (Exception  e) {
            log.error("【执行命令失败】\n执行的命令如下：\n" + cmd + "\n" + e.getMessage());
            e.printStackTrace();
        }

        return buffer.toString();
    }



    /**
     * 登录主机
     * @return
     *  登录成功返回true，否则返回false
     */
    public static Connection login(String ip, String userName, String password){
        boolean isAuthenticated = false;
        Connection conn = null;
        long startTime = Calendar.getInstance().getTimeInMillis();
        try {
            conn = new Connection(ip);
            // 连接主机
            conn.connect();
            // 认证
            isAuthenticated = conn.authenticateWithPassword(userName, password);
            if(isAuthenticated){
                System.out.println(String.format(tipStr, "认证成功"));
            } else {
                System.out.println(String.format(tipStr, "认证失败"));
            }
        } catch (IOException e) {
            System.err.println(String.format(tipStr, "登录失败"));
            e.printStackTrace();
        }
        long endTime = Calendar.getInstance().getTimeInMillis();
        System.out.println("登录用时: " + (endTime - startTime)/1000.0 + "s\n" + splitStr);
        return conn;
    }

    /**
     * 远程执行shell脚本或者命令
     * @param cmd
     *  即将执行的命令
     * @return
     *  命令执行完后返回的结果值
     */
    public static String execute(Connection conn, String cmd){
        String result = "";
        Session session = null;
        try {
            if(conn != null){
                // 打开一个会话
                session = conn.openSession();
                // 执行命令
                session.execCommand(cmd);
                result = processStdout(session.getStdout(), DEFAULT_CHAR_SET);
                //如果为得到标准输出为空，说明脚本执行出错了
                if(StringUtils.isBlank(result)){
                    System.err.println("【得到标准输出为空】\n执行的命令如下：\n" + cmd);
                    result = processStdout(session.getStderr(), DEFAULT_CHAR_SET);
                }else{
                    System.out.println("【执行命令成功】\n执行的命令如下：\n" + cmd);
                }
            }
        } catch (IOException e) {
            System.err.println("【执行命令失败】\n执行的命令如下：\n" + cmd + "\n" + e.getMessage());
            e.printStackTrace();
        } finally {
            if (conn != null) {
                conn.close();
            }
            if (session != null) {
                session.close();
            }
        }
        return result;
    }

    /**
     * 解析脚本执行返回的结果集
     * @param in 输入流对象
     * @param charset 编码
     * @return
     *  以纯文本的格式返回
     */
    private static String processStdout(InputStream in, String charset){
        InputStream stdout = new StreamGobbler(in);
        StringBuffer buffer = new StringBuffer();
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(stdout, charset));
            String line = null;
            while((line = br.readLine()) != null){
                buffer.append(line + "\n");
            }
        } catch (UnsupportedEncodingException e) {
            System.err.println("解析脚本出错：" + e.getMessage());
            e.printStackTrace();
        } catch (IOException e) {
            System.err.println("解析脚本出错：" + e.getMessage());
            e.printStackTrace();
        }
        return buffer.toString();
    }
}
